
#ifndef _POSTGRESQL_LIBPQ_6_2_X__
#define _POSTGRESQL_LIBPQ_6_2_X__
#include <cstdlib>
#include <libpq-fe.h>
#include <mpcl/string.hh>
#include <mpcl/extended_map.hh>
#include "uesql_base_manager.hh"


///  Name-space for UESQL.
namespace uesql
{

  namespace postgresql_6_2_x
  {
    ///  Type with connection with host, database and handle triple.
    struct TConnectionTriple
    {

      public:

	PGconn*         ptPgconn;
	mpcl::xstring   yDatabaseName;
	mpcl::xstring   yHostName;
        mpcl::xstring   yUserName;
        mpcl::xstring   yUserPassword;


      public:

	//
	//  C O N S T R U C T O R S
	//

	TConnectionTriple (void) :
	  ptPgconn      (NULL)   ,
	  yDatabaseName ()       ,
	  yHostName     ()       ,
          yUserName     ()       ,
          yUserPassword ()       {}

	TConnectionTriple ( const mpcl::xstring& rkyHOST_NAME     ,
			    const mpcl::xstring& rkyDATABASE_NAME ,
			    const mpcl::xstring& rkyUSER_NAME     ,
			    const mpcl::xstring& rkyUSER_PASSWORD ,
			    PGconn*                  ptPGCONN         ) :
	  ptPgconn      (ptPGCONN)                                      ,
	  yDatabaseName (rkyDATABASE_NAME)                              ,
	  yHostName     (rkyHOST_NAME)                                  ,
          yUserName     (rkyUSER_NAME)                                  ,
          yUserPassword (rkyUSER_PASSWORD)                              {}

    };  // struct TConnectionTriple
   
  }  // namespace postgresql_6_2_x

  ///  PostgreSQL LIBPQ 6.2.x manager class.
  class TPostgresql62xManager : 
    public TUesqlBaseManager<postgresql_6_2_x::TConnectionTriple, PGresult*>
  {
  
    public:
  
      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TPostgresql62xManager (void)                                                :
        TUesqlBaseManager<postgresql_6_2_x::TConnectionTriple, PGresult*>() {}

      ///  Connects to current database and host.
      void connect (const mpcl::xstring& rkyCONNECTION_NAME)
      {
        connect ( rkyCONNECTION_NAME               ,
                  tCurrentConnection.yHostName     ,
                  tCurrentConnection.yDatabaseName ,
                  tCurrentConnection.yUserName     ,
                  tCurrentConnection.yUserPassword );
      }

      ///  Connects to current host.
      void connect ( const mpcl::xstring& rkyCONNECTION_NAME ,
                     const mpcl::xstring& rkyDATABASE_NAME   )
      {
        connect ( rkyCONNECTION_NAME               ,
                  tCurrentConnection.yHostName     ,
                  rkyDATABASE_NAME                 ,
                  tCurrentConnection.yUserName     ,
                  tCurrentConnection.yUserPassword );
      }

      void connect ( const mpcl::xstring& rkyCONNECTION_NAME ,
                     const mpcl::xstring& rkyHOST_NAME       ,
                     const mpcl::xstring& rkyDATABASE_NAME   ,
                     const mpcl::xstring& rkyUSER_LOGIN      )
      {
        mpcl::xstring   yUserName;
        mpcl::xstring   yUserPassword;
        
        if ( rkyUSER_LOGIN.find ('/') == mpcl::xstring::npos )
        {
          yUserName = rkyUSER_LOGIN;
          yUserName.suppress ('\'');
        }
        else
        {
          yUserName     = rkyUSER_LOGIN.substr (0, rkyUSER_LOGIN.find ('/'));
          yUserPassword = rkyUSER_LOGIN.substr (rkyUSER_LOGIN.find ('/') + 1);
          yUserName.suppress ('\'');
          yUserPassword.suppress ('\'');
        }
        connect ( rkyCONNECTION_NAME ,
                  rkyHOST_NAME       ,
                  rkyDATABASE_NAME   ,
                  yUserName          ,
                  yUserPassword      );
      }

      void connect ( const mpcl::xstring& rkyCONNECTION_NAME ,
                     const mpcl::xstring& rkyHOST_NAME       ,
                     const mpcl::xstring& rkyDATABASE_NAME   ,
                     const mpcl::xstring& rkyUSER_NAME       ,
                     const mpcl::xstring& rkyUSER_PASSWORD   );

      /**
      *  Frees statement execution result \a ptPGRESULT.
      *  @param ptPGRESULT Source result.
      */
      void clearResult (PGresult* ptPGRESULT)
      {
        if ( ptPGRESULT )
        {
          PQclear (ptPGRESULT);
        }
      }

      /**
      *  Diagnoses statement result (\a ptPGRESULT).
      *  It updates properly SQL status variables.
      *  @param ptPGRESULT Source statement execution result.
      */
      void diagnose (PGresult* ptPGRESULT);

      /**
      *  Diagnoses connection sanity (first stage diagnostics). It
      *  updates properly SQL status variables.
      *  @param rtCONNECTION_TRIPLE Connection to check.
      */
      void diagnoseConnection (postgresql_6_2_x::TConnectionTriple& rtCONNECTION_TRIPLE);

      void diagnoseSingleRow (PGresult* ptPGRESULT);

      void disconnect (const mpcl::xstring kyCONNECTION_NAME);
  
      PGresult* execute (const char* pkcSTATEMENT);

      PGresult* execute (const mpcl::xstring& rkySTATEMENT);

      PGresult* execute (const char* pkcSTATEMENT, const char* pkcCONNECTION_NAME);

      void initialize (void);
  
  };  // class TPostgresql62xManager


  //
  //  E X P O R T E D   V A R I A B L E S
  //

  extern TPostgresql62xManager   htPostgresql62xManager;

}  // namespace uesql


#endif  // not _POSTGRESQL_LIBPQ_6_2_X__
